@@ -170,6 +170,7 @@ proxy_mode_radiobutton_clicked_cb (GtkWidget *widget,
 {
 	GSList *mode_group;
 	int mode;
+	GConfClient *client;
 	
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget)))
 		return;
@@ -184,6 +185,10 @@ proxy_mode_radiobutton_clicked_cb (GtkWidget *widget,
 				  mode == PROXYMODE_MANUAL);
 	gtk_widget_set_sensitive (WID ("auto_box"),
 				  mode == PROXYMODE_AUTO);
+	client = gconf_client_get_default ();
+	gconf_client_set_bool (client, USE_PROXY_KEY,
+				  mode == PROXYMODE_AUTO || mode == PROXYMODE_MANUAL, NULL);
+	g_object_unref (client);
 }
 
 static void
@@ -287,6 +292,10 @@ main (int argc, char **argv)
 	client = gconf_client_get_default ();
 	gconf_client_add_dir (client, "/system/gnome-vfs",
 			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+	gconf_client_add_dir (client, "/system/http_proxy",
+			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+	gconf_client_add_dir (client, "/system/proxy",
+			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
 	dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-network-preferences.glade",
 				"network_dialog", NULL);
